/*
** Set v1.0
**
** A WWW value setting script
**
** Copyright (c) 27.3.95 by Andreas Ley <ley@rz.uni-karlsruhe.de>
**
** Permission to use, copy, modify, and distribute this software for any
** purpose and without fee is hereby granted, provided that the above
** copyright notice appears in all copies. This software is provided "as is"
** and without any express or implied warranties.
**
** This program has been tested on a HP9000/715 with HP-UX A.09.05
** In this environment, neither lint -u nor gcc -Wall produce any messages.
** If you encounter any errors or need to make any changes to port it
** to another platform, please contact me.
**
** Version 1.0 - 27.3.95
**	Initial version
*/

char header[]="set v1.0\n(c) 1995 by Andreas Ley\n";

/* General definitions */

#define	MASTER	"webmaster@rz.uni-karlsruhe.de"

/* File locations */

#define	ADDRESSES	"/usr/local/etc/httpd/config/mailto.conf"

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "set_german.h"

#define	FALSE	0		/* This is the naked Truth */
#define	TRUE	1		/* and this is the Light */

/* Functions from util.c */

char *makeword(char *line, char stop);
char *fmakeword(FILE *f, char stop, int *len);
void unescape_url(char *url);
void plustospace(char *str);

/* Global variables */

char	master[]=MASTER;


/* Prints header of a HTML-Request */
void show_header(char *subtitle, char *description)
{
	(void)printf("Content-type: text/html\n\n");
	(void)printf("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n");
	(void)printf("<HTML>\n");
	(void)printf("<HEAD>\n");
	(void)printf("<TITLE>%s%s%s</TITLE>\n",TITLE,subtitle?" - ":"",subtitle?subtitle:"");
	(void)printf("<LINK REV=\"made\" HREF=\"mailto:%s\">\n",master);
	(void)printf("</HEAD>\n");
	(void)printf("<BODY>\n");
	(void)printf("<H1>%s</H1>\n",subtitle?subtitle:TITLE);
	if (description) {
		(void)printf(description,master);
		(void)printf("<P>\n");
	}
}



/* Prints trailer of a HTML-Request */
void show_trailer()
{
	(void)printf("</BODY>\n");
	(void)printf("</HTML>\n");
}



/* Shows an error page, based on wrong user input */
void show_error(char *error)
{
	show_header(ERROR,error);
	show_trailer();
	exit(0);
}



/* Shows an error page, based on internal misfunction */
void show_fatal(char *error)
{
	show_header(ERROR,error);
	(void)printf(ERROR_FATAL,master);
	(void)printf("<P>\n");
	show_trailer();
	exit(0);
}



/* Returns a Location: header */
void show_location(char *location)
{
	(void)printf("Location: %s\n",location);
	(void)printf("URI: %s\n\n",location);
}



char *secure(char *text)
{
	char	*src,*dest;

	if (text) {
		for (src=text,dest=text;*src;src++)
			if ((*src&0x7f)>=0x20)
				*dest++=*src;
		*dest='\0';
	}
	return(text);
}



void usage(char *image)
{
	(void)fprintf(stderr,"Usage: %s [-h] [-v]\n",image);
	exit(1);
}


int main(int argc, char *argv[])
{
	int		c;
	char		buffer[1024],*fname,*text,*ptr,*sptr,*nptr,*keep;
	char		*tag,*val,*htag[1024],*hval[1024];
	int		cl,cnt,max,fd;
	size_t		len;
	struct stat	statbuf;

	while ((c=getopt(argc,argv,"vh?")) != EOF)
		switch ((char)c) {
		case 'v':
			(void)fprintf(stderr,header);
			exit(0);
		case 'h':
			(void)fprintf(stderr,header);
		case '?':
			usage(argv[0]);
		}

	if (strcmp(getenv("REQUEST_METHOD"),"POST"))
		show_fatal(ERROR_REQUEST_METHOD);
	if (strcmp(getenv("CONTENT_TYPE"),"application/x-www-form-urlencoded")) {
		(void)sprintf(buffer,ERROR_CONTENT_TYPE,getenv("CONTENT_TYPE"),"application/x-www-form-urlencoded");
		show_fatal(buffer);
	}

	cl=atoi(getenv("CONTENT_LENGTH"));
	if (cl==0)
		show_fatal(ERROR_CONTENT_LENGTH);
	max=0;
	while (cl&&(!feof(stdin))) {
		val=fmakeword(stdin,'&',&cl);
		plustospace(val);
		unescape_url(val);
		tag=secure(makeword(val,'='));
		if (*val) {
			htag[max]=tag;
			hval[max]=val;
			max++;
		}
	}

	fname=getenv("PATH_TRANSLATED");
	fd=open(fname,O_RDONLY);
	if (fd<0) {
		sprintf(buffer,ERROR_READ,fname,strerror(errno));
		show_fatal(buffer);
	}
	fstat(fd,&statbuf);
	len=statbuf.st_size;
	text=malloc(len);
	read(fd,text,len);
	close(fd);

	fd=open(fname,O_WRONLY|O_TRUNC);
	if (fd<0) {
		sprintf(buffer,ERROR_WRITE,fname,strerror(errno));
		show_fatal(buffer);
	}
	while (text&&(ptr=strstr(text,"<!--#set"))) {
		write(fd,text,ptr-text);
		ptr+=8;
		if (!(strncmp(ptr,"#keep",5))) {
			keep=ptr-8;
			ptr+=5;
		}
		else
			keep=NULL;
		while (*ptr==' ')
			ptr++;
		sptr=strstr(ptr,"-->");
		while (ptr<sptr) {
			switch (*ptr) {
				case '{':
					write(fd,"<",1);
					break;
				case '}':
					write(fd,">",1);
					break;
				case '$':
					if ((nptr=strchr(ptr+1,'$'))) {
						*nptr='\0';
						for (cnt=0;cnt<max;cnt++)
							if (!(strcmp(htag[cnt],ptr+1)))
								break;
						if (cnt<max)
							write(fd,hval[cnt],strlen(hval[cnt]));
						*nptr='$';
						ptr=nptr;
						break;
					}
				case '\\':
					ptr++;
				default:
					write(fd,ptr,1);
					break;
			}
			ptr++;
		}
		text=strchr(sptr,'\n');
		if (keep)
				write(fd,keep,text-keep);
		if (text&&*(text-1)=='\r')
			text--;
	}
	if (text)
		write(fd,text,strlen(text));
	close(fd);

	sprintf(buffer,"http://%s%s",getenv("SERVER_NAME"),getenv("PATH_INFO"));
	show_location(buffer);

	return(0);
}
